/*----------------------------------------------------------------------------------
*
* COPYRIGHT (c) 2001 by Singing Electrons, Inc. All rights reserved.
*
* Module Name			: C:\se\adi\hidclass\source\main.c
*
* Description			: The Main entrypoint
*
*
* Revision History	: At bottom of the file.
*
*---------------------------------------------------------------------------------*/

/** include files **/
#include <sys/exception.h>
#include <sys/excause.h>
#include <defBF535.h>
#include "dtype.h"
#include "usb.h"
#include "usbdriver.h"
#include "hidclass.h"
#include "dprintf.h"

/** local definitions **/
#define DisplayError //Later on we will do something special to display the error?


//This is for Eagle-35 only!
#define kVal 0xff00
static inline void initLEDs(void) {
  *(volatile uint16 *)FIO_DIR = (uint16)kVal;
  asm("ssync;");
}

static inline void setLEDs(void) {
  *(volatile uint16 *)FIO_FLAG_S = (uint16)kVal;
  asm("ssync;");
}


static inline void clearLEDs(void) {
  *(volatile uint16 *)FIO_FLAG_C = (uint16)kVal;
  asm("ssync;");
} 

static inline uint16 getPushBtns(void) {
  uint16 val = *(volatile uint16 *)FIO_FLAG_S;
  asm("ssync;");
  return val;
}
#define	DELAY_LOOP 300000






#define VENDOR_ID 0xCB00
#define PRODUCT_ID 0x0005
#define DEVICE_RELEASE 0x0005 //0.05

#if CREATE_A_MOUSE
#define EP1_REPORT_SIZE 3 //3 bytes
#define USE_IN_ENDPOINT 1

char gButtonStates = 0;
char gXMovement = 1;
char gYMovement = 1;

//
// HID Report Descriptor - for a 3 button Mouse
//
UCHAR gTheReportDescriptor[] =
	{
		HID_USAGE_PAGE, HID_USAGE_PAGE_DESKTOP,
		HID_USAGE, HID_USAGE_MOUSE,
		HID_BEGIN_COLLECTION, HID_COLLECTION_APPLICATION,
			HID_USAGE, HID_USAGE_POINTER,
			HID_BEGIN_COLLECTION, HID_COLLECTION_PHYSICAL,
				HID_USAGE_PAGE, HID_USAGE_PAGE_BUTTONS,
				HID_USAGE_MINIMUM, 0x01,
				HID_USAGE_MAXIMUM, 0x03,
				HID_LOGICAL_MINIMUM, 0,
				HID_LOGICAL_MAXIMUM, 1,
				HID_REPORT_COUNT, 3, //3 buttons
				HID_REPORT_SIZE, 1, // 1 bit each
				HID_INPUT, HID_DATA_VARIABLE_ABSOLUTE,
				HID_REPORT_COUNT, 1,
				HID_REPORT_SIZE, 5, //3 bits "stuffed"
				HID_INPUT, HID_CONSTANT,
				HID_USAGE_PAGE, HID_USAGE_PAGE_DESKTOP,
				HID_USAGE, HID_USAGE_X,
				HID_USAGE, HID_USAGE_Y,
				HID_LOGICAL_MINIMUM, 0x81,
				HID_LOGICAL_MAXIMUM, 0x7F,
				HID_REPORT_SIZE, 8, //8 bit each
				HID_REPORT_COUNT, 2, //2 axes
				HID_INPUT, HID_DATA_VARIABLE_RELATIVE,
			HID_END_COLLECTION,
		HID_END_COLLECTION,
	};


#else //CREATE_A_MOUSE

#define EP1_REPORT_SIZE 1 //1 byte
#define USE_IN_ENDPOINT 0

//
// HID Report Descriptor - for a non-specific device
//
UCHAR gTheReportDescriptor[] =
	{
		HID_USAGE_PAGE_LONG, LOBYTE(HID_USAGE_PAGE_MYOWN), HIBYTE(HID_USAGE_PAGE_MYOWN),
		HID_USAGE, HID_USAGE_IODEVICE,
		HID_BEGIN_COLLECTION, HID_COLLECTION_APPLICATION,
				HID_USAGE_MINIMUM, 0x01,
				HID_USAGE_MAXIMUM, 0x08,
				HID_LOGICAL_MINIMUM, 0,
				HID_LOGICAL_MAXIMUM, 1,
				/*
				HID_REPORT_SIZE, 1, //1 bit each
				HID_REPORT_COUNT, 8, //1 bytes
				HID_INPUT, HID_DATA_VARIABLE_ABSOLUTE,

				HID_USAGE_MINIMUM, 0x01,
				HID_USAGE_MAXIMUM, 0x08,
				HID_OUTPUT, HID_DATA_VARIABLE_ABSOLUTE,

				HID_USAGE_MINIMUM, 0x01,
				HID_USAGE_MAXIMUM, 0x08,
				*/
				HID_REPORT_SIZE, 1, //1 bit each
				HID_REPORT_COUNT, 8, //1 bytes
				HID_FEATURE, HID_DATA_VARIABLE_ABSOLUTE,
		HID_END_COLLECTION,
	};

#endif //CREATE_A_MOUSE



/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/
static void delay(void);

/** public data **/

/** private data **/
int dumpDprintf = 0;

/** public functions **/

/** private functions **/
void UpdateReport (void);

/** callbacks **/
static void OnReportSent (void);
static void OnNAKSent (void);
static bool OnGetReportRequest (uint16 reportTypeAndID, void *buffer, PUINT pNumBytesToTransfer);
static bool OnSetReportRequest (uint16 reportTypeAndID, void *buffer, UINT numBytesInReport);
static void OnSuspend (void);
static void OnResume (void);
static void OnReset (void);


/*------------------------------------------------------------
*	main
*
*	Parameters:  
*		None
*
*	Globals Used:
*		None
*
*	Description:
*		This is the entry-point. 
*
*	Returns:
*		Nothing
*
*
*------------------------------------------------------------*/
void main (void)
	{
		InitDprintf ();



		//Init HIDCLASS - which will in turn setup the USBDriver
#if CREATE_A_MOUSE
		if (!HIDCLASS_Init (VENDOR_ID, PRODUCT_ID, DEVICE_RELEASE, 
			true /* useINEP */, gTheReportDescriptor, sizeof (gTheReportDescriptor),
			nil /*OnGetReportRequest*/, nil /*OnSetReportRequest*/,
			OnReportSent, OnNAKSent, 
			OnSuspend, OnResume, OnReset))
				goto done;
#else
		if (!HIDCLASS_Init (VENDOR_ID, PRODUCT_ID, DEVICE_RELEASE, 
			false /* useINEP */, gTheReportDescriptor, sizeof (gTheReportDescriptor),
			OnGetReportRequest, OnSetReportRequest,
			nil /*OnReportSent*/, nil /*OnNAKSent*/, 
			OnSuspend, OnResume, OnReset))
				goto done;
#endif

		//printf ("Init complete\n");
		//A real application would at this point do something!
	  initLEDs(); 
    clearLEDs();  

	  while(1) 
	  	{
#if CREATE_A_MOUSE
		    clearLEDs();  
		    delay();
		    setLEDs();
		    delay();
#endif //CREATE_A_MOUSE

				if (dumpDprintf)
					{
						DumpDprintf (dumpDprintf);
						dumpDprintf = 0;
					}
		  }

done:
		//Run happily ever after
		while (1)
			;
				
	}


void delay(void)
{
  unsigned long i;
  
  for(i=0; i<DELAY_LOOP; i++)
    {
#if CREATE_A_MOUSE
			if ((i & 0x7fff) == 0)
				UpdateReport ();
#endif //CREATE_A_MOUSE
      asm("nop;nop;nop;nop;");
    }
}





#if CREATE_A_MOUSE
/*------------------------------------------------------------
*	UpdateReport
*
*	Parameters:  
*
*	Globals Used:
*		gButtonStates, gXMovement, gYMovement
*
*	Description:
*		Updates the global variables used to prepare the HID report.
*		In this case, it reads the pushbuttons on the Eagle-35 board
*		and prepares the variables required by the HID Report preparation
*		routine.
*
*	Returns:
*		Nothing
*
*------------------------------------------------------------*/
void UpdateReport (void)
	{
		uint16 pushButtons = getPushBtns ();
		gButtonStates = pushButtons & 0x07; //first three buttons map to mouse buttons
		gXMovement = 0;
		gYMovement = 0;
		if (pushButtons & 0x08) //Fourth button is +1 X
			gXMovement = 1;
		if (pushButtons & 0x10) //Fifth button is +1 X
			gXMovement = -1;
		if (pushButtons & 0x20) //Sixth button is +1 Y
			gYMovement = 1;
		if (pushButtons & 0x40) //Seventh button is -1 Y
			gYMovement = -1;
	}


/*------------------------------------------------------------
*	OnReportSent
*
*	Parameters:  
*
*	Globals Used:
*		gButtonStates, gXMovement, gYMovement
*
*	Description:
*		This routine is called when an HID report has been successfully
*		sent, or when the device is configured. It prepares data for
*		the next IN request and submits it to HIDCLASS.
*
*	Returns:
*
*
*------------------------------------------------------------*/
void OnReportSent (void)
	{
		UCHAR reportData[EP1_REPORT_SIZE];

		reportData[0] = gButtonStates; //Button information
		reportData[1] = gXMovement;//X movement
		reportData[2] = gYMovement;//Y movement

		HIDCLASS_PrepareReportEndpoint (reportData, EP1_REPORT_SIZE);

	}



/*------------------------------------------------------------
*	OnNAKSent
*
*	Parameters:  
*
*	Globals Used:
*		gButtonStates, gXMovement, gYMovement
*
*	Description:
*		This routine is called when the HIDCLASS finds out that a NAK
*		was sent on the IN endpoint because it did not have report data
*		ready. This routine prepares the data for the next IN request
*		and submits it to HIDCLASS.
*
*	Returns:
*
*
*------------------------------------------------------------*/
void OnNAKSent (void)
	{
		UCHAR reportData[EP1_REPORT_SIZE];

		reportData[0] = gButtonStates; //Button information
		reportData[1] = gXMovement;//X movement
		reportData[2] = gYMovement;//Y movement

		HIDCLASS_PrepareReportEndpoint (reportData, EP1_REPORT_SIZE);

	}

#else //CREATE_A_MOUSE



/*------------------------------------------------------------
*	OnGetReportRequest
*
*	Parameters:  
*
*	Globals Used:
*		None
*
*	Description:
*		This routine is called when the control endpoint receives
*		a Get Report (HID) request. This routine is responsible for
*		filling out the data in the buffer that will be sent back. In
*		this case it builds the report data based on the push button
*		states on the Eagle-35.
*
*	Returns:
*		true to "succeed" the request, false to cause a stall
*
*------------------------------------------------------------*/
bool OnGetReportRequest (uint16 reportTypeAndID, void *buffer, PUINT pNumBytesToTransfer)
	{
		UCHAR report[EP1_REPORT_SIZE];

		report[0] = (UCHAR)(getPushBtns() & 0xff);
		if (*pNumBytesToTransfer < EP1_REPORT_SIZE)
			memcpy (buffer, &report, *pNumBytesToTransfer);
		else
			{
				memcpy (buffer, &report, sizeof (report));
				*pNumBytesToTransfer = sizeof (report);
			}

		return(true);
		
	}



/*------------------------------------------------------------
*	OnSetReportRequest
*
*	Parameters:  
*
*	Globals Used:
*		None
*
*	Description:
*		This routine is called when the control endpoint receive
*		a Set Report (HID) request. This routine uses the data
*		supplied in the buffer. In this case it uses the provided
*		data to set and clear the LEDs on the Eagle-35
*
*	Returns:
*		true to "succeed" the request, false to cause a stall
*
*------------------------------------------------------------*/
bool OnSetReportRequest (uint16 reportTypeAndID, void *buffer, UINT numBytesInReport)
	{
		UCHAR report[EP1_REPORT_SIZE];
		if (numBytesInReport >= EP1_REPORT_SIZE)
			{
				memcpy (&report, buffer, sizeof (report));
				clearLEDs();
				*(volatile uint16 *)FIO_FLAG_S = ((uint16)report[0] << 8);
			}
		return(true);
	}
#endif //CREATE_A_MOUSE


/*------------------------------------------------------------
*	OnSuspend
*
*	Parameters:  
*
*	Globals Used:
*		None
*
*	Description:
*		This routine will be called when USBDriver detects a
*		suspend condition. Currently, it does nothing.
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnSuspend (void)
	{
	}



/*------------------------------------------------------------
*	OnResume
*
*	Parameters:  
*
*	Globals Used:
*		None
*
*	Description:
*		This routine will be called when the USBDriver detects that
*		the bus is out of suspend. Currently, it does nothing for custom
*		device. For the mouse, it prepares the IN endpoint with the report.
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnResume (void)
	{

#if CREATE_A_MOUSE
		UCHAR reportData[EP1_REPORT_SIZE];

		reportData[0] = gButtonStates; //Button information
		reportData[1] = gXMovement;//X movement
		reportData[2] = gYMovement;//Y movement

		HIDCLASS_PrepareReportEndpoint (reportData, EP1_REPORT_SIZE);
#endif //CREATE_A_MOUSE

	}


/*------------------------------------------------------------
*	OnReset
*
*	Parameters:  
*
*	Globals Used:
*		None
*
*	Description:
*		This routine will be called when the USBDriver detects reset
*		signalling on the bus. Currently, it does nothing
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnReset (void)
	{
	}


/*----------------------------------------------------------------------------------
* $Log: main.c,v $
* Revision 1.1  2002/10/31 00:30:59  Devendra
* Moved all files in one folder to avoid IDE related problems.
*
* Revision 1.3  2002/10/30 02:36:40  Devendra
* - Added more abstraction to HIDCLASS. So that it's possible to build
*   different devices without making any changes to the HIDCLASS module.
* - Added proper support for handling USB SUSPEND, RESUME, and RESET events.
*
* Revision 1.2  2002/10/22 17:47:47  Devendra
* - Refine LED blinking, etc.
* - Change output location.
*
* Revision 1.1  2002/10/22 17:23:39  Devendra
* Rearranged file locations.
*
* Revision 1.6  2002/10/14 05:16:04  Devendra
* - Custom Device now using Feature reports for I/O
* - Added compiler define/project configuration to switch between mouse and custom device.
*
* Revision 1.5  2002/10/14 01:33:15  Devendra
* Got "custom" device functional with buttons and lights!
*
* Revision 1.4  2002/10/09 17:09:32  Devendra
* - Added support for handling IN endpoints.
* - Added HID descriptors (HID and HID Report)
* - Modified the Interface descriptors to comply to HID Class Spec.
* - Added functionality to use the push-buttons on the Eagle-35 as a mouse (for buttons as well as for movement).
* - The device is now a fully functioning USB Mouse!
*
* Revision 1.3  2002/10/08 20:44:03  Devendra
* - Separated the printf logging to a different c module.
*
* Revision 1.2  2002/10/08 19:57:02  Devendra
* - Added specific callbacks for getting Device, Config, and String descriptors.
* - BugFix: Max. Packet Size was not being set correctly in ArmEndpoint.
* - Added internal buffer based logging to allow real-time "printfs".
* - Added String descriptors.
*
* Revision 1.1  2002/09/20 06:51:30  Devendra
* First Rev.
*
*
*---------------------------------------------------------------------------------*/
